@@ -1530,7 +1530,8 @@ gcm_prefs_profiles_treeview_clicked_cb (GtkTreeSelection *selection, gpointer us
 	clut = gcm_profile_generate_curve (profile, 256);
 
 	/* only show if there is useful information */
-	size = gcm_clut_get_size (clut);
+	if (clut != NULL)
+		size = gcm_clut_get_size (clut);
 	if (size > 0) {
 		g_object_set (trc_widget,
 			      "clut", clut,
@@ -1645,8 +1646,7 @@ gcm_prefs_profiles_treeview_clicked_cb (GtkTreeSelection *selection, gpointer us
 	}
 
 	/* set delete sensitivity */
-	ret = (filename != NULL && g_str_has_prefix (filename, "/home/"));
-	egg_debug ("filename: %s", filename);
+	ret = gcm_profile_get_can_delete (profile);
 	widget = GTK_WIDGET (gtk_builder_get_object (builder, "button_profile_delete"));
 	gtk_widget_set_sensitive (widget, ret);
 
@@ -2377,7 +2377,7 @@ gcm_prefs_startup_phase1_idle_cb (gpointer user_data)
 	/* setup RGB combobox */
 	widget = GTK_WIDGET (gtk_builder_get_object (builder, "combobox_space_rgb"));
 	colorspace_rgb = gconf_client_get_string (gconf_client, GCM_SETTINGS_COLORSPACE_RGB, &error);
-	if (colorspace_rgb == NULL) {
+	if (colorspace_rgb == NULL && error != NULL) {
 		egg_warning ("failed to get configuration value: %s", error->message);
 		g_clear_error (&error);
 	}
@@ -2389,7 +2389,7 @@ gcm_prefs_startup_phase1_idle_cb (gpointer user_data)
 	/* setup CMYK combobox */
 	widget = GTK_WIDGET (gtk_builder_get_object (builder, "combobox_space_cmyk"));
 	colorspace_cmyk = gconf_client_get_string (gconf_client, GCM_SETTINGS_COLORSPACE_CMYK, NULL);
-	if (colorspace_cmyk == NULL) {
+	if (colorspace_cmyk == NULL && error != NULL) {
 		egg_warning ("failed to get configuration value: %s", error->message);
 		g_clear_error (&error);
 	}
@@ -2401,8 +2401,8 @@ gcm_prefs_startup_phase1_idle_cb (gpointer user_data)
 	/* setup rendering lists */
 	widget = GTK_WIDGET (gtk_builder_get_object (builder, "combobox_rendering_display"));
 	gcm_prefs_set_combo_simple_text (widget);
-	intent_display = gconf_client_get_string (gconf_client, GCM_SETTINGS_RENDERING_INTENT_DISPLAY, NULL);
-	if (intent_display == NULL) {
+	intent_display = gconf_client_get_string (gconf_client, GCM_SETTINGS_RENDERING_INTENT_DISPLAY, &error);
+	if (intent_display == NULL && error != NULL) {
 		egg_warning ("failed to get configuration value: %s", error->message);
 		g_clear_error (&error);
 	}
@@ -2412,8 +2412,8 @@ gcm_prefs_startup_phase1_idle_cb (gpointer user_data)
 
 	widget = GTK_WIDGET (gtk_builder_get_object (builder, "combobox_rendering_softproof"));
 	gcm_prefs_set_combo_simple_text (widget);
-	intent_softproof = gconf_client_get_string (gconf_client, GCM_SETTINGS_RENDERING_INTENT_SOFTPROOF, NULL);
-	if (intent_softproof == NULL) {
+	intent_softproof = gconf_client_get_string (gconf_client, GCM_SETTINGS_RENDERING_INTENT_SOFTPROOF, &error);
+	if (intent_softproof == NULL && error != NULL) {
 		egg_warning ("failed to get configuration value: %s", error->message);
 		g_clear_error (&error);
 	}
@@ -2422,7 +2422,7 @@ gcm_prefs_startup_phase1_idle_cb (gpointer user_data)
 			  G_CALLBACK (gcm_prefs_renderer_combo_changed_cb), (gpointer) "softproof");
 
 	/* coldplug plugged in devices */
-	ret = gcm_client_add_connected (gcm_client, &error);
+	ret = gcm_client_add_connected (gcm_client, GCM_CLIENT_COLDPLUG_ALL, &error);
 	if (!ret) {
 		egg_warning ("failed to add connected devices: %s", error->message);
 		g_error_free (error);
